-------------------------------------------------------------------------------
--
-- Design unit:   Control block
--                 
--
-- File name:     ctrl.vhd
--
-- Description:   Control block is fundamental block of HECTOR evaluation 
--                platform, it consists of two components: ahb_reg and ram_if. 
--                Ahb_reg operates registers which are available from MSS. 
--                Ram_if ensure data transfer from/to external RAM. Control 
--                block uses these two components to operate with user entity 
--                and transfer data to PC host. A lot of signals and registers 
--                are not used, they are used only as a template for making 
--                user adjustable system. 
--
--                This user adjustable system consists of two tools, the first 
--                one is a command execution state machine (CMD_EXE) and the 
--                second one is user application logic. 
--
--                CMD_EXE executes command when new data are written to the CMD
--                register. Commands can be triggered by Fab packet (packet 
--                writes data to command and fabin register). New command can be 
--                added by creating of new command definition (constant) and its 
--                inserting to CMD_EXE.
--
--                User logic is adjustable. In this example, user logic loads data 
--                from RAM memory, proccess them and save back to the RAM memory. 
--                Amount of data depends on wsize_reg register - size of required 
--                file.
--
-- System:        VHDL'93
--
-- Author:        Marek Laban, MICRONIC a.s., LabHC
--
-- Copyright:     MICRONIC a.s., LabHC
--
-- Created in frame of HECTOR project (No. 644052).
--
-- Revision:      Version 1.00, May 2016
-- History:       16/05/2016 - v1.00 Created (ML, MIC)
--
-------------------------------------------------------------------------------


LIBRARY IEEE;

USE ieee.std_logic_unsigned.ALL;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;

ENTITY ctrl IS
  PORT (
  -----------------------------------------------------------------------------
  -- AHB MASTER bus (external RAM) --------------------------------------------
  -----------------------------------------------------------------------------
    htrans     : OUT STD_LOGIC_VECTOR(1  DOWNTO 0);   
    hsize      : OUT STD_LOGIC_VECTOR(1  DOWNTO 0);
    hwrite     : OUT STD_LOGIC;    
    hwdata     : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);   
    haddr      : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
    hmastlock  : OUT STD_LOGIC;            
    hsel       : OUT STD_LOGIC;  
    hreadyout  : IN  STD_LOGIC; 
    hresp      : IN  STD_LOGIC;       
    hrdata     : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    hburst     : OUT STD_LOGIC_VECTOR(2  DOWNTO 0);
    --
  -----------------------------------------------------------------------------
  -- AHB SLAVE bus (MSS)-------------------------------------------------------
  -----------------------------------------------------------------------------
    haddr_mss  : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    hwdata_mss : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    hrdata_mss : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
    hsel_mss   : IN  STD_LOGIC;   
    hready_mss : OUT STD_LOGIC;   
    hwrite_mss : IN  STD_LOGIC;
    --
  -----------------------------------------------------------------------------
  -- Control block signals and registers --------------------------------------
  -----------------------------------------------------------------------------
  -- global signals 
    n_rst      : IN  STD_LOGIC;
    clk        : IN  STD_LOGIC;
  -- signals to process unit
    data_out   : OUT STD_LOGIC_VECTOR(255 downto 0);
    data_in    : IN  STD_LOGIC_VECTOR(255 downto 0);
    address_out : OUT STD_LOGIC_VECTOR( 7 downto 0);
    data_out_cipher   : OUT STD_LOGIC_VECTOR(127 downto 0);
    data_in_cipher    : IN  STD_LOGIC_VECTOR(127 DOWNTO 0);
    strt_proc  : OUT STD_LOGIC;                       -- start process
    end_proc   : IN  STD_LOGIC;                       -- process end
    txt_format : OUT STD_LOGIC;                       -- format of text
    WExSO      : OUT STD_LOGIC;                       -- cipher block write enable
    CSxSO      : OUT STD_LOGIC;                       -- cipher block chip select
  -- other signals
    LED_out    : OUT STD_LOGIC

  );
END ctrl;

ARCHITECTURE architecture_ctrl OF ctrl IS

  -- BEGIN COMPONENT DEFINITIONS **********************************************

  COMPONENT ram_if IS
  PORT(
  -- global reset and clock signals
    n_rst            : IN    STD_LOGIC;                  
    clk              : IN    STD_LOGIC;                  
  ---------------------------------------------------------------------------- 
  -- AHB bus signals ---------------------------------------------------------
  ----------------------------------------------------------------------------
    -- AHB-Lite transfer type for master. 
    -- Indicates the type of the current transfer
    htrans            : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);   
    -- AHB-Lite transfer size for master. Indicates the size of the transfer. 
    -- Only byte, half-word and word transactions are supported
    hsize             : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);
    -- AHB-Lite write indication for master 0. 
    -- High for a write, Low for a read.           
    hwrite            : OUT   STD_LOGIC;    
    -- AHB-Lite write data for master                   
    hwdata            : OUT   STD_LOGIC_VECTOR(31 DOWNTO 0);   
    -- AHB-Lite address
    haddr             : OUT   STD_LOGIC_VECTOR(31 DOWNTO 0);
    -- AHB-Lite locked sequence indication for master 0      
    hmastlock         : OUT   STD_LOGIC;            
    -- AHB-Lite slave 0 select                     
    hsel              : OUT   STD_LOGIC;  
    -- AHB-Lite ready signal from slave 0                    
    hreadyout         : IN    STD_LOGIC; 
    -- AHB-Lite transfer response for master                     
    hresp             : IN    STD_LOGIC;       
    -- AHB-Lite read data for master              
    hrdata            : IN    STD_LOGIC_VECTOR(31 DOWNTO 0);
    -- AHB-Lite burst register output
    hburst            : OUT   STD_LOGIC_VECTOR(2 DOWNTO 0);
    --
  ----------------------------------------------------------------------------
  -- Interface for R/W data to/from external RAM memmory ---------------------
  ----------------------------------------------------------------------------
    -- data input and valid input data signal
    data_in           : IN    STD_LOGIC_VECTOR(31 DOWNTO 0);
    data_in_vld       : IN    STD_LOGIC;  -- data are valid during asserting 1 
    -- data output and valid output data signal
    data_out          : OUT   STD_LOGIC_VECTOR(31 DOWNTO 0);
    data_out_vld      : OUT   STD_LOGIC;  -- data are valid during asserting 1
    --
    -- start address of DMA 
    addr_in           : IN    STD_LOGIC_VECTOR(31 DOWNTO 0);
    -- bytes count (number of 8-bit words (rounded to multiple of 4))
    byte_cnt          : IN    STD_LOGIC_VECTOR(31 DOWNTO 0);
    --
    -- control signals  
    data_wr           : IN    STD_LOGIC;  -- data read(0) or data write(1) 
    ram_strt          : IN    STD_LOGIC;  -- ram start (strobe during 1 clk period)
    -- info signals  
    wr_buf_fl         : OUT   STD_LOGIC;  -- write buffer full (1 when full)
    buff_ofw          : OUT   STD_LOGIC;  -- buffer overflow (1 when overflow)
    ram_bsy           : OUT   STD_LOGIC;  -- ram busy (1 when busy, 0 when idle) 
    -- actual address of ram opertion
    ram_stg           : OUT   STD_LOGIC_VECTOR(31 DOWNTO 0) 
  );
  END COMPONENT;
  
  COMPONENT ahb_reg IS
  PORT (
  -- global signals 
    n_rst      : IN    STD_LOGIC;                  
    clk        : IN    STD_LOGIC;
    --
  -- AHB SLAVE bus (MSS)
    haddr_mss  : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    hwdata_mss : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    hrdata_mss : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
    hsel_mss   : IN  STD_LOGIC;   
    hready_mss : OUT STD_LOGIC;   
    hwrite_mss : IN  STD_LOGIC;
    -- 
  -- registers available from MSS
    cmd_reg    : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  -- command register
    state_reg  : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);  -- state of operation
    stage_reg  : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);  -- actual used address
    --
    fabin_reg  : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  -- fabric input (from MSS)
    fabout_reg : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);  -- fabric output (to MSS)
    fabst_reg  : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);  -- fabric status (to MSS)
    --
    -- parameters where will be saved data for user file generation 
    -- (RAM to MSS file)
    wstart_reg : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  -- start address
    wsize_reg  : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  -- size in bytes
    --
    -- parameters where will be loaded user file (MSS file to RAM)
    rstart_reg : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  -- start address
    rsize_reg  : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  -- size in bytes
    --
    user1_reg  : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  -- user optional registers
    user2_reg  : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);   
    --
  -- new command received signal (assert high)
    new_cmd    : OUT STD_LOGIC
  -----------------------------------------------------------------------------
  );
  END COMPONENT;

  -- END COMPONENT DEFINITIONS ************************************************


  -- BEGIN COMMAND ************************************************************
  -- command definitions for the control block  
    -- Idle mode (nothing to do)
    CONSTANT MODE_IDLE      : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"00000000";
    -- Operation start !! fixed for MSS !!
    CONSTANT CMD_OP_ST      : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"00000001";
    -- Set daughter board 1 active (HDMI) 
    CONSTANT CMD_SETDB1     : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"00000002";
    -- Set daughter board 2 active (SATA) 
    CONSTANT CMD_SETDB2     : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"00000003";
    -- set lowercase text format command
    CONSTANT CMD_TOLOWR     : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"00000004";
    -- set uppercase text format command
    CONSTANT CMD_TOUPPR     : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"00000005";
    -- User command 0 (not used)
    CONSTANT CMD_USER0      : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"00000007"; 
    -- User command 1 (not used)
    CONSTANT CMD_USER1      : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"00000008";  


  -- command execution signals and registers 
    -- command state machine states
    TYPE cmd_state_type IS( CMD_IDLE,
                            CMD_EXE
                           ); 
    SIGNAL cmd_state        : cmd_state_type; 
    -- new command was received signal
    SIGNAL new_cmd          : STD_LOGIC;       
    -- command register
    SIGNAL cmd_reg          : STD_LOGIC_VECTOR(31 DOWNTO 0);  
    -- fabric input register (from MSS)
    SIGNAL fabin_reg        : STD_LOGIC_VECTOR(31 DOWNTO 0);
    -- fabric output register (to MSS)
    SIGNAL fabout_reg       : STD_LOGIC_VECTOR(31 DOWNTO 0);
    -- fabric status register (to MSS)
    SIGNAL fabst_reg        : STD_LOGIC_VECTOR(31 DOWNTO 0);
    --
    -- multiplexing between daughter boards (0 - HDMI, 1 - SATA connector)
    SIGNAL mux_out          : STD_LOGIC;      
  -- END COMMAND ***************************************************************
  


  -- BEGIN TEMPLATE DESIGN SIGNALS ********************************************* 
  -- state of acquistion register definitions  
    -- acquisition idle state
    CONSTANT OP_IDLE_STATE  : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"FFFFFFFF"; 
    -- busy state
    CONSTANT OP_BUSY_STATE  : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"00000001"; 
    -- buffer overflow state
    CONSTANT OP_OFLW_STATE  : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"00000002"; 
    -- buffer overflow state
    CONSTANT OP_ERR_STATE   : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"00000003"; 
    --
    -- start address of write ram interface operation
    SIGNAL wstart_reg       : STD_LOGIC_VECTOR(31 DOWNTO 0);  
    -- size in bytes of write ram interface operation
    SIGNAL wsize_reg        : STD_LOGIC_VECTOR(31 DOWNTO 0);  
    -- start address of read ram interface operation
    SIGNAL rstart_reg       : STD_LOGIC_VECTOR(31 DOWNTO 0);  
    -- size in bytes of read ram interface operation
    SIGNAL rsize_reg        : STD_LOGIC_VECTOR(31 DOWNTO 0);  
    --
    -- user optional register
    SIGNAL user1_reg        : STD_LOGIC_VECTOR(31 DOWNTO 0);  
    -- user optional register    
    SIGNAL user2_reg        : STD_LOGIC_VECTOR(31 DOWNTO 0);  
    --    
    -- operation state register
    SIGNAL state_reg        : STD_LOGIC_VECTOR(31 DOWNTO 0);  
    --
    -- operation state machine states
    TYPE op_state_type IS     ( OP_IDLE,
                                START_LOAD_DATA,
                                LOAD_DATA,
                                PROCESS_DATA,
                                WAIT_FOR_CIPHER,
                                START_SAVE_DATA,
                                SAVE_DATA
                              ); 
    SIGNAL op_state     : op_state_type;
    --
    -- operation enable signal
    SIGNAL op_enable    : STD_LOGIC;   
    --              
    -- data input of ram interface
    SIGNAL ram_in       : STD_LOGIC_VECTOR(31 DOWNTO 0); 
    -- valid data input of ram interface
    SIGNAL ram_in_vld   : STD_LOGIC;    
    --
    -- data output of ram interface
    SIGNAL ram_out      : STD_LOGIC_VECTOR(31 DOWNTO 0); 
    -- valid data ouput of ram interface
    SIGNAL ram_out_vld  : STD_LOGIC;    
    --
    -- start address of ram interface operation
    SIGNAL ram_addr     : STD_LOGIC_VECTOR(31 DOWNTO 0); 
    -- number of bytes of ram interface operation           
    SIGNAL ram_cnt      : STD_LOGIC_VECTOR(31 DOWNTO 0); 
    --
    -- ram interface type of operation (1 - write, 0 - read)
    SIGNAL ram_rw       : STD_LOGIC;    
    -- ram interface start signal (assert 1 during one clk period)
    SIGNAL ram_strt     : STD_LOGIC;
    --
    -- read buffer not empty  
    SIGNAL rd_buf_ne    : STD_LOGIC;
    -- write buffer full  
    SIGNAL wr_buf_fl    : STD_LOGIC;
    -- write buffer overflow occurs  
    SIGNAL ram_ofw      : STD_LOGIC;
    -- 
    -- actual used address of ram interface
    SIGNAL stage_reg    : STD_LOGIC_VECTOR(31 DOWNTO 0);
    -- ram interface busy (1 - r/w operation occurs) 
    SIGNAL ram_bsy      : STD_LOGIC;
  -- END TEMPLATE DESIGN SIGNALS ***********************************************
    --
    -- SIGNALS AND REGISTERS USED IN EXAMPLE ------------
    -- start address used for data reading
    SIGNAL rstart_addr  : STD_LOGIC_VECTOR(31  DOWNTO 0); 
    -- start address used for data writing
    SIGNAL wstart_addr  : STD_LOGIC_VECTOR(31  DOWNTO 0);
    -- number of written bytes
    SIGNAL wbytes       : STD_LOGIC_VECTOR(31  DOWNTO 0);  
    -- actual of loaded words
    SIGNAL act_words    : STD_LOGIC_VECTOR(2   DOWNTO 0);
    -- buffer where are data loaded and processed
    SIGNAL my_word      : STD_LOGIC_VECTOR(255 DOWNTO 0);

    SIGNAL CSxS          : STD_LOGIC;
    SIGNAL WExS          : STD_LOGIC;
    SIGNAL StartCipherxS : STD_LOGIC;
    SIGNAL CipherDonexS  : STD_LOGIC;

    -- operation state machine states
    TYPE cipher_state_type IS   ( CIPHER_IDLE,
                                CIPHER_LOAD_KEY_I,
                                CIPHER_LOAD_KEY,
                                CIPHER_LOAD_PT_I,
                                CIPHER_LOAD_PT,
                                CIPHER_READ_CT
                              ); 
    SIGNAL cipher_state     : cipher_state_type;

    -- SIGNAL data_from_cipher : STD_LOGIC_VECTOR(127 DOWNTO 0);
    SIGNAL data_to_cipher   : STD_LOGIC_VECTOR(127 DOWNTO 0);
    SIGNAL CiphertextxDI    : STD_LOGIC_VECTOR(127 DOWNTO 0);
    SIGNAL TagxDI           : STD_LOGIC_VECTOR(127 DOWNTO 0);

BEGIN

  i_ram_if: ram_if 
  PORT MAP (
  -- global signals  
    n_rst      => n_rst,
    clk        => clk,

  -- AHB signals and registers (every is led out to upper entity) --
    htrans     => htrans, 
    hsize      => hsize, 
    hwrite     => hwrite,
    hwdata     => hwdata,
    haddr      => haddr,
    hmastlock  => hmastlock,
    hsel       => hsel,
    hreadyout  => hreadyout,
    hresp      => hresp,
    hrdata     => hrdata,
    hburst     => hburst,
    --
  -- user interface signals --
    data_in      => ram_in,
    data_in_vld  => ram_in_vld,
    --
    data_out     => ram_out,
    data_out_vld => ram_out_vld,
    --
    addr_in    => ram_addr,
    byte_cnt   => ram_cnt,
    --
    data_wr    => ram_rw,
    ram_strt   => ram_strt,
    --
    wr_buf_fl  => wr_buf_fl,
    buff_ofw   => ram_ofw,
    --
    -- ram_stg   => , --> not used
    ram_bsy    => ram_bsy
  );


  i_ahb_reg: ahb_reg
  PORT MAP (
  -- global signals  
    n_rst      => n_rst,
    clk        => clk,
    --
  -- AHB SLAVE bus (MSS) 
    haddr_mss  => haddr_mss,
    hwdata_mss => hwdata_mss,
    hrdata_mss => hrdata_mss,
    hsel_mss   => hsel_mss,   
    hready_mss => hready_mss,   
    hwrite_mss => hwrite_mss,
    --
  -- registers available from MSS
    cmd_reg    => cmd_reg,
    state_reg  => state_reg,
    stage_reg  => stage_reg,
    -- registers available from user instructions (UART)
    fabin_reg  => fabin_reg, 
    fabout_reg => fabout_reg,
    fabst_reg  => fabst_reg,
    --
    wstart_reg => wstart_reg,
    wsize_reg  => wsize_reg,
    rstart_reg => rstart_reg,
    rsize_reg  => rsize_reg,
    -- user optional registers
    user1_reg  => user1_reg,
    user2_reg  => user2_reg,
    --
  -- new command received signal (assert high)
    new_cmd    => new_cmd
  ); 

--***************************************************************************--
--*************** COMMAND execution state machine template ******************--
--***************************************************************************--
  PROCESS(clk, n_rst)
  BEGIN
    IF (n_rst = '0') THEN
      cmd_state  <= CMD_IDLE;
      mux_out    <= '0';
      txt_format <= '0';
    ELSIF rising_edge(clk) THEN
      CASE cmd_state IS
        WHEN CMD_IDLE =>              -- command awaiting
          IF (new_cmd = '1') THEN
            cmd_state <= CMD_EXE;
          END IF;
        WHEN CMD_EXE  =>              -- command execution
          CASE cmd_reg IS
            WHEN CMD_OP_ST =>         -- operation is started from MSS
              cmd_state <= CMD_IDLE;  -- by inserting of this command
            WHEN CMD_SETDB1 =>        -- set daughter board 1 as active board
              mux_out <= '0';         -- (HDMI)
              cmd_state <= CMD_IDLE;
            WHEN CMD_SETDB2 =>        -- set daughter board 2 as active board
              mux_out <= '1';         -- (SATA)
              cmd_state <= CMD_IDLE;
            WHEN CMD_TOUPPR =>        -- set uppercase text format
              txt_format <= '1';
              cmd_state <= CMD_IDLE;
            WHEN CMD_TOLOWR =>        -- set lowercase text format
              txt_format <= '0';
              cmd_state <= CMD_IDLE;
            --
            WHEN CMD_USER0 =>         -- you can insert your own code here
              cmd_state <= CMD_IDLE;  
            WHEN CMD_USER1 =>
              cmd_state <= CMD_IDLE;
            WHEN OTHERS => NULL;
          END CASE;
        WHEN OTHERS => NULL;
      END CASE;
    END IF;
  END PROCESS;

--***************************************************************************--


--***************************************************************************--
--****************** USER APP LOGIC TEMPLATE DESIGN *************************--
--***************************************************************************--

  PROCESS(clk, n_rst)
  BEGIN
    IF (n_rst = '0') THEN
      op_state      <= OP_IDLE;
      ram_strt      <= '0';  
      ram_in_vld    <= '0';
      strt_proc     <= '0';
      StartCipherxS <= '0';
      data_out      <= (OTHERS => '0');
    ELSIF rising_edge(clk) THEN
      CASE op_state IS
        WHEN OP_IDLE =>
          IF(op_enable = '1') THEN 
            -- load start address where was file written by MSS
            rstart_addr <= rstart_reg;               
            -- load start address where will be data written to RAM
            wstart_addr <= wstart_reg;
            -- load size of stream which will be processed
            wbytes      <= wsize_reg;
            op_state    <= START_LOAD_DATA;
          END IF;    

        -- load data from external RAM (initialization)
        WHEN START_LOAD_DATA =>
          IF(ram_bsy = '0')THEN
            ram_addr    <= rstart_addr;  -- set initial address
            ram_cnt     <= x"00000020";  -- load 8 words (32 bytes)
            --
            ram_rw      <= '0';          -- set read from RAM
            -- 
            ram_strt    <= '1';          -- start RAM operation
            --
            act_words   <= (OTHERS =>'0'); 
            op_state    <= LOAD_DATA;
          END IF;
        
        -- load data from external RAM  
        WHEN LOAD_DATA =>                    
          ram_strt <= '0';    
          --
          -- save data when they are valid on data output
          IF(ram_out_vld = '1')THEN  
            --
            my_word    <= my_word(223 downto 0) & ram_out;
            act_words  <= act_words + 1;
            -- when whole buffer is filled process data
            IF(act_words = "111")THEN
              op_state <= PROCESS_DATA;
            END IF;
            --
          END IF;

        WHEN PROCESS_DATA =>
          data_out  <= my_word;
          -- data_out  <= x"1122334455667788991222324252627282921323334353637383931424344454";
          strt_proc <= '1';
          IF(end_proc = '1')THEN
            strt_proc <= '0';
            my_word   <= data_in;
            StartCipherxS <= '1';
            -- op_state  <= START_SAVE_DATA;
            op_state  <= WAIT_FOR_CIPHER;
          END IF;
        -- save data to external RAM (initialization)

        WHEN WAIT_FOR_CIPHER =>
          StartCipherxS <= '1';
          IF(CipherDonexS = '1')THEN   
            my_word <= TagxDI & CiphertextxDI;
            StartCipherxS <= '0';
            op_state  <= START_SAVE_DATA;            
          END IF;
 
        -- save data to external RAM (initialization)
        WHEN START_SAVE_DATA =>
          IF(ram_bsy = '0')THEN
            ram_addr  <= wstart_addr;
            -- check how much bytes should be written
            IF(wbytes >= x"00000020")THEN
              ram_cnt <= x"00000020";
              -- decrement written bytes
              wbytes    <= wbytes - x"00000020";
            ELSE
              ram_cnt   <= wbytes;
              -- decrement written bytes
              wbytes    <= (OTHERS =>'0');
            END IF;

             -- enable writting to RAM
            ram_rw    <= '1';
            -- enable RAM interfae
            ram_strt  <= '1';
            op_state  <= SAVE_DATA;
          END IF;

        -- save data to external RAM 
        WHEN SAVE_DATA =>
          ram_strt <= '0';
          -- while buffer is not full -> fulfill buffer - don't care about 
          -- how much are inserted to buffer, because RAM interface has defined 
          -- number of bytes to write and at the end of RAM interface operation
          -- buffer is flushed
          IF(wr_buf_fl = '0')THEN 
            -- insert data to RAM interface
            ram_in     <= my_word (255 downto 224);
            -- set new world for buffer
            my_word    <= my_word (223 downto 0) & (x"00000000");
            -- set valid data signal
            ram_in_vld <= '1';    
          ELSE
            ram_in_vld <= '0';
          END IF;
          --
          -- wait while ram is busy
          IF(ram_bsy = '0')THEN

            -- if nothing to write - stop process
            IF(wbytes = x"00000000")THEN
              op_state    <= OP_IDLE;

            -- else load data again
            ELSE
              -- increment read and write start address
              wstart_addr <= wstart_addr + x"00000020";
              rstart_addr <= rstart_addr + x"00000020";
              op_state    <= START_LOAD_DATA;
            END IF;
          END IF;

      END CASE;
    END IF;
  END PROCESS; 


--***************************************************************************--
--****************** CIPHER INTERFACE TEMPLATE DESIGN ***********************--
--***************************************************************************--

  PROCESS(clk, n_rst)
  BEGIN
    IF (n_rst = '0') THEN
      cipher_state  <= CIPHER_IDLE;
      data_to_cipher       <= (OTHERS => '0');
      address_out   <= (OTHERS => '0');  
      WExS          <= '0';
      CSxS          <= '0';
      CipherDonexS  <= '0';
    ELSIF rising_edge(clk) THEN
      CipherDonexS  <= '0';
      CASE cipher_state IS
        WHEN CIPHER_IDLE =>
            data_to_cipher      <= (OTHERS => '0');
            address_out   <= (OTHERS => '0');  
            WExS          <= '0';
            CSxS          <= '0';
            IF(StartCipherxS = '1') THEN
                cipher_state    <= CIPHER_LOAD_PT_I;
            END IF;    
        -- load data
        WHEN CIPHER_LOAD_PT_I =>
            address_out   <= x"02";  
            WExS          <= '1';
            CSxS          <= '1';            
            data_to_cipher  <= my_word(127 downto 0);                                  
            cipher_state    <= CIPHER_LOAD_PT;
        WHEN CIPHER_LOAD_PT =>
            address_out   <= x"08";  
            data_to_cipher  <= my_word(127 downto 0); --x"00000000000000000000000000000000";                                  
            WExS          <= '1';
            CSxS          <= '1';
            cipher_state    <= CIPHER_READ_CT;
        -- read ciphertext
        WHEN CIPHER_READ_CT =>
            address_out   <= x"0c";  
            WExS          <= '0';
            CSxS          <= '1';            
            data_to_cipher  <= x"00000000000000000000000000000000";                  
            CiphertextxDI <= data_in_cipher;
            CipherDonexS <= '1';
            cipher_state    <= CIPHER_IDLE;
      END CASE;
    END IF;
  END PROCESS; 


  CSxSO <= CSxS;
  WExSO <= WExS;

  data_out_cipher <= data_to_cipher;

  -- stage register is available from MSS
  WITH op_state SELECT 
  state_reg <= OP_IDLE_STATE   WHEN OP_IDLE,
               OP_BUSY_STATE   WHEN OTHERS;
 
  -- operation is enabled when start operation command is inserted
  op_enable <= '1' WHEN ( (cmd_state = CMD_EXE  ) AND 
                          (cmd_reg   = CMD_OP_ST)     ) ELSE '0';

  -- stage register is available from MSS
  stage_reg <= wstart_addr;  -- monitor actual address
  LED_out   <= op_enable;            


  -- not used signals and registers
  fabout_reg   <= (OTHERS =>'0');
  fabst_reg    <= (OTHERS =>'0');
  user2_reg    <= (OTHERS =>'0');
  
--***************************************************************=************--

END architecture_ctrl;
