-------------------------------------------------------------------------------
-- Title      : Bus logic of Ketje module
-- Project    : 
-------------------------------------------------------------------------------
-- File       : ketje_fast_bus.vhdl
-- Author     : Filippo Melzani
-- Company    : 
-- Created    : 2017-01-30
-- Last update: 2017-01-30
-- Platform   : 
-- Standard   : VHDL'93/02

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity ketje is
  
  port (
    ClkxCI       : in  std_logic;
    RstxRBI      : in  std_logic;
    CSxSI        : in  std_logic;       -- active-high chip select
    WExSI        : in  std_logic;       -- active-high write enable
    AddressxDI   : in  std_logic_vector(7 downto 0);
    DataWritexDI : in  std_logic_vector(127 downto 0);
    DataReadxDO  : out std_logic_vector(127 downto 0));

end entity ketje;

architecture structural of ketje is

    signal r_data_in : std_logic_vector(127 downto 0);
    signal r_data_out : std_logic_vector(127 downto 0);

    signal din         : std_logic_vector(31 downto 0);
    signal din_size    : std_logic_vector(2 downto 0);
    signal go          : std_logic;
    signal soft_reset  : std_logic;
    signal suv         : std_logic;
    signal auth_data   : std_logic;
    signal data        : std_logic;
    signal decrypt     : std_logic;
    signal tag         : std_logic;
    signal tag_p_one   : std_logic;
    signal last        : std_logic;
    signal hash        : std_logic;
    signal squeeze     : std_logic;
    signal dout        : std_logic_vector(31 downto 0);
    signal dout_valid  : std_logic;
    signal ready       : std_logic;
  
    type main_state is (idle, in0, in1, in2, in3, run, out0, out1, out2, out3);
    signal main_st : main_state;
 
begin

    din_size    <= "000";
    go <= '1' when main_st=in0 or main_st=in1 or main_st=in2 or main_st=in3 else '0';
    
    din <=  r_data_in(127 downto 96) when main_st=in0 else
            r_data_in( 95 downto 64) when main_st=in1 else
            r_data_in( 63 downto 32) when main_st=in2 else
            r_data_in( 31 downto  0);
    
    process(ClkxCI, RstxRBI)
    begin
        if (RstxRBI = '1') then
            soft_reset  <= '0';
            suv         <= '0';
            auth_data   <= '0';
            data        <= '0';
            decrypt     <= '0';
            tag         <= '0';
            tag_p_one   <= '0';
            last        <= '0';
            hash        <= '0';
            squeeze     <= '0';
            r_data_in   <= (others=>'0');
            r_data_out  <= (others=>'0');
            main_st     <= idle;
            DataReadxDO <= (others=>'0');
        elsif rising_edge(ClkxCI) then
            main_st <= idle;
            if CSxSI='1' then
                if WExSI='1' then
                    -- Writing
                    if AddressxDI=X"02" then
                        soft_reset  <= DataWritexDI(9);
                        suv         <= DataWritexDI(8);
                        auth_data   <= DataWritexDI(7);
                        data        <= DataWritexDI(6);
                        decrypt     <= DataWritexDI(5);
                        tag         <= DataWritexDI(4);
                        tag_p_one   <= DataWritexDI(3);
                        last        <= DataWritexDI(2);
                        hash        <= DataWritexDI(1);
                        squeeze     <= DataWritexDI(0);
                    elsif AddressxDI=X"08" then
                        r_data_in <= DataWritexDI;
                        main_st <= in0;
                    end if;
                else
                    -- Reading
                    if AddressxDI=X"01" then
                        DataReadxDO <= (others=>'0');
                        DataReadxDO(0) <= ready;
                    elsif AddressxDI=X"02" then
                        DataReadxDO <= (others=>'0');
                        DataReadxDO(9) <= soft_reset;
                        DataReadxDO(8) <= suv;
                        DataReadxDO(7) <= auth_data;
                        DataReadxDO(6) <= data;
                        DataReadxDO(5) <= decrypt;
                        DataReadxDO(4) <= tag;
                        DataReadxDO(3) <= tag_p_one;
                        DataReadxDO(2) <= last;
                        DataReadxDO(1) <= hash;
                        DataReadxDO(0) <= squeeze;
                    elsif AddressxDI=X"0C" then
                        DataReadxDO  <= r_data_out;
                    end if;
                end if;
            else
                if main_st=in0 then
                    main_st<=in1;
                elsif main_st=in1 then
                    main_st<=in2;
                elsif main_st=in2 then
                    main_st<=in3;
                elsif main_st=in3 then
                    main_st<=run;
                elsif main_st=run then
                    if dout_valid='0' then
                        main_st <= run;
                    else
                        main_st<=out0;
                        r_data_out(127 downto 96) <= dout;
                    end if;
                elsif main_st=out0 then
                    main_st<=out1;
                    r_data_out(95 downto 64) <= dout;
                elsif main_st=out1 then
                    main_st<=out2;
                    r_data_out(63 downto 32) <= dout;
                elsif main_st=out2 then
                    main_st<=out3;
                    r_data_out(31 downto 0) <= dout;
                else
                    main_st<=idle;
                end if;
            end if;
        end if;
    end process;
        
  ketje_core_1: entity work.ketjesrv2
        port map(   clk         => ClkxCI,
                    rst         => RstxRBI,
                                
                    din         => din,
                    din_size    => din_size,
                    go          => go,
                    
                    soft_reset  => soft_reset,
                    suv         => suv,
                    auth_data   => auth_data,
                    data        => data,
                    decrypt     => decrypt,
                    tag         => tag,
                    tag_p_one   => tag_p_one,
                    last        => last,
                                   
                    hash        => hash,
                    squeeze     => squeeze,
                    -- output      -- output
                    dout        => dout,
                    dout_valid  => dout_valid,
                    ready       => ready
                );

end structural;
